WGo.TreeView = (function() {
	"use strict";
	
	var TreeView = function(player, elem) {
		this.element = elem;
		this.tree_wrapper = document.createElement("div");
		this.tree_wrapper.className = "tree2 coordinates";
		elem.appendChild(this.tree_wrapper);
		
		this.size = 19;
		this.player = player;
		this.root = [];
		
		this.grid = [];
		
		player.addEventListener("kifuLoaded", this.create.bind(this));
		player.addEventListener("update", this.update.bind(this));
	}

	TreeView.A_CODE = "A".charCodeAt(0);
	TreeView.MARGIN = 10;
	TreeView.SIZE = 25;

	TreeView.prototype = {
		constructor: TreeView,
		
		save_to_grid: function(row, col, val, branch) {
			var shift = 0;
			
			// at first we check field if it is free, otherwise we find first empty 
			while(this.grid[row+shift] && this.grid[row+shift][col]) shift++;
			
			// prepare row
			this.grid[row+shift] = this.grid[row+shift] || [];
			
			// when wanted field isn't free we must shift the whole branch up
			if(shift > 0) {
			
				// rewrite branch in the grid (it ends with edge)
				var shifted_col = col-1;
				while(this.grid[row][shifted_col] && this.grid[row][shifted_col].className != "edge") {
					this.grid[row][shifted_col].style.top = (row + shift)*(TreeView.SIZE + TreeView.MARGIN)+"px";
					this.grid[row+shift][shifted_col] = this.grid[row][shifted_col];
					this.grid[row][shifted_col] = undefined;
					
					shifted_col--;
				}
				
				// rewrite edges in the grid
				for(var i = 1; i <= shift; i++) {
					this.grid[row+i][shifted_col] = this.grid[row][shifted_col];
				}
				this.grid[row][shifted_col].style.height = (parseInt(this.grid[row][shifted_col].style.height) + shift*(TreeView.SIZE + TreeView.MARGIN))+"px";
				
				// shift branch DOM elements
				/*for(var i = 0; i < branch.length; i++) {
					branch[i].style.top = (row + shift)*(TreeView.SIZE + TreeView.MARGIN)+"px";
				}*/
				
				// edge as well
				/*if(branch.edge) {
					branch.edge.style.height = (parseInt(branch.edge.style.height) + shift*(TreeView.SIZE + TreeView.MARGIN))+"px";
				}*/
			}
			
			// save to the grid
			this.grid[row+shift][col] = val;
			
			return row+shift;
		},
		
		process_node: function(kifu_node, parent_ul, branch, path, row, col) {
			
			// create node li element
			var li = document.createElement("li");
			li.addEventListener("click", this.player.goTo.bind(this.player, path));
			li.style.left = ((col-1) * (TreeView.SIZE+TreeView.MARGIN))+"px";
			li._kifu_node = kifu_node;
						
			if(kifu_node.move) {
				if(kifu_node.move.c == WGo.B) li.className = "black";
				else  li.className = "white";
				
				// when node contais move, we modify li
				if(kifu_node.move.pass) {
					li.title = "pass";
				}
				else {
					var letter = kifu_node.move.x;
					if(kifu_node.move.x > 7) letter++;
					letter = String.fromCharCode(letter+TreeView.A_CODE);
					
					li.title = letter+""+(this.size-kifu_node.move.y);
				}
				
				li.innerHTML = path.m;
			}
			else {
				// otherwise we add class 'empty'
				li.className = "empty";
			}
			
			// if node contains comment, we add another class
			if(kifu_node.comment) {
				li.className += " commented";
			}
			
			// append to the DOM
			parent_ul.appendChild(li);
			
			// save internally
			branch.push(li);		
			
			// save to the grid (it is possible, that branch needs to be shifted, in that case row will be changed)
			row.i = this.save_to_grid(row.i, col, li);		
			
			li.style.top = (row.i * (TreeView.SIZE + TreeView.MARGIN))+"px";
			
			if(!kifu_node.children || !kifu_node.children.length) return;
			else if(kifu_node.children.length == 1) {
				// if node has one children we process it
				// clone path object and increase move number
				var new_path = {};
				for(var key in path) new_path[key] = path[key];
				new_path.m++;
				
				this.process_node(kifu_node.children[0], parent_ul, branch, new_path, row, col+1);
			}
			else {
				// if node has more children we have to make fork
				var fork = [];
				branch.push(fork);
				
				// create DOM
				var li = document.createElement("li");
				li.className = "fork";
				parent_ul.appendChild(li);
				fork.li = li;
				
				// process children one by one
				for(var i in kifu_node.children) {
					
					// clone path object and update path
					var new_path = {};
					for(var key in path) new_path[key] = path[key];
					new_path.m++;
					new_path[new_path.m] = i;
					
					this.process_branch(kifu_node.children[i], li, fork, new_path, i == 0 ? row : {i:row.i+1}, col);
				}
			}
		},
		
		process_branch: function(first_node, elem, parent_fork, path, row, col) {
			
			// create new branch
			var branch = [];
			
			// create ul element for storing that branch
			var ul = document.createElement("ul");
			branch.ul = ul;
			
			// prepare row in grid
			this.grid[row.i] = this.grid[row.i] || [];
			
			if(parent_fork.length) {
				// if this branch isn't first of its parent, we have to worry about edges
				
				if(this.grid[row.i][col]) {
					// if field in grid is not free, we must shift branch up(down)
					while(this.grid[row.i] && this.grid[row.i][col]) row.i++;
					
					// prepare new row in grid
					this.grid[row.i] = this.grid[row.i] || [];
				}
				
				// add edge to the DOM
				var edge = document.createElement("div");
				edge.className = "edge";
				edge.style.left = ((col-1)*(TreeView.SIZE+TreeView.MARGIN) + TreeView.SIZE/2)+"px";
				elem.appendChild(edge);
				branch.edge = edge;
				
				if(parent_fork.length == 1) {
					// 2nd branch has different edge than others
					edge.style.top = ((row.i)*(TreeView.SIZE+TreeView.MARGIN) - TreeView.MARGIN)+"px";
					edge.style.height = (TreeView.SIZE-2)+"px";
				}
				else {
					edge.style.top = ((row.i)*(TreeView.SIZE+TreeView.MARGIN) - TreeView.MARGIN - TreeView.SIZE/2)+"px";
					edge.style.height = (TreeView.SIZE+TreeView.MARGIN)+"px";	
				}
				
				// save edge to the grid
				this.grid[row.i][col] = edge;
			}
			else {
				// first branch is active
				ul.className = "active";
			}
			
			// add branch and ul to the tree
			parent_fork.push(branch);
			elem.appendChild(ul);
			
			// process first node of the branch
			this.process_node(first_node, ul, branch, path, row, col+1);
		},
		
		create: function(ev) {
			// prepare environment
			this.size = ev.kifu.size;
			this.root = [];
			this.grid = [];
			this.tree_wrapper.innerHTML = "";
			
			if(ev.kifu.nodeCount !== undefined && ev.kifu.nodeCount > 2000) {
				this.no_update = true;
				this.tree_wrapper.innerHTML = '<div class="error">Too many nodes to display game tree...</div>';
				return;
			}
			
			// process main branch
			this.process_branch(ev.kifu.root, this.tree_wrapper, this.root, {m:0}, {i:0}, 0);
			this.no_update = false;
		},
		
		update: function(ev) {
			if(this.no_update) return;
			
			// remove old active class
			var old = this.tree_wrapper.querySelector("li.active");
			if(old) old.className = old.className.replace(/(?:^|\s)active(?!\S)/g, "");
			
			// current move number
			var move = 0;
			
			// curent position within a branch
			var position = 0;
			
			// current branch
			var branch = this.root[0];
			var variation;
			
			while(move < ev.path.m) {
				// increase move number
				move++;
				// increase position
				position++;
				
				// if there is fork, change branch
				if(branch[position] && branch[position].constructor == Array) {
					// select following variation (branch)
					variation = ev.path[move] || 0;
					
					// remove all 'active' classes from fork's ul
					for(var i = 0; i < branch[position].length; i++) {
						branch[position][i].ul.className = branch[position][i].ul.className.replace(/(?:^|\s)active(?!\S)/g, "");
					}
					
					// add 'active' class to wanted ul 
					branch[position][variation].ul.className += " active";
					
					// update branch and position
					branch = branch[position][variation];
					position = 0;
				}
			}
			
			// and we have actual node
			branch[position].className += " active";
			
			// make node visible
			this.scroll_into_view(branch[position]);
		},
		
		scroll_into_view: function(elem) {
			if(this.element.scrollLeft > elem.offsetLeft) {
				this.element.scrollLeft = elem.offsetLeft;
			}
			else if(this.element.scrollLeft < elem.offsetLeft - this.element.offsetWidth + 30 + elem.offsetWidth) {
				this.element.scrollLeft = elem.offsetLeft - this.element.offsetWidth + 30 + elem.offsetWidth;
			}

			if(this.element.scrollTop > elem.offsetTop) {
				this.element.scrollTop = elem.offsetTop;
			}
			else if(this.element.scrollTop < elem.offsetTop - this.element.offsetHeight + 30 + elem.offsetHeight) {
				this.element.scrollTop = elem.offsetTop - this.element.offsetHeight + 30 + elem.offsetHeight;
			}
		},
		
		displayNumbers: function() {
			this.tree_wrapper.className = this.tree_wrapper.className.replace(/(?:^|\s)coordinates(?!\S)/g, "");
		},
		
		displayCoordinates: function() {
			this.tree_wrapper.className = this.tree_wrapper.className.replace(/(?:^|\s)coordinates(?!\S)/g, "");
			this.tree_wrapper.className += " coordinates";
		},
		
		addNode: function(node, path) {
			// todo: for performance
		}
	}
	
	
	return TreeView;
})();
