/*
Supported specific gobook attributs are :
	gb_show_size
	gb_play_size
	gb_from_part
	gb_to_part
	gb_language

for sgf content only :
	gb_vw
	gb_ca

*/

// Some other ideas (not implemented yet)
// - sgf to figure
// - sgf mode, that will allow to display sgf with variations as figure (with only the main variation, or even with a tree viewer ?)
// - allow (or not) to place stones
// - show a game by x moves.
// - when passing sgf data, customize detection of correct vs wrong variations.

function initGoBooks(selector,config) {
	if (!config) {
		config = {};
	}
	$(selector).each (function(i,e) {
		
		
		var content = $(e).html().replace(/^[\t| ]+/gm, '').trim(); // remove left spaces and tabs...
		
		if (content.indexOf("(;") === 0) {
			// transform SGF tree to gobook problem
			var vw = $(e).attr("gb_vw"); // optional
			var ca = $(e).attr("gb_ca"); // optional
			content = GBProblemHelper.sgfToPrb(content,{vw : vw, ca : ca });
			//console.log(content);
		}

		if (content.indexOf("::book") !== 0) {
			content = "::book\n\n"+content;
		}

		$(e).html("");
		
		var engine = new GoBookEngine();
		var res = new GoBookParser().parse(content);
		var book = res.book;
		
		// look into the console to see the json structure of the book
		//console.log(JSON.stringify(book,null,2));
		var bridge=null;
		if ($(e).attr("gb_show_size")) {
			config_new =  JSON.parse(JSON.stringify(config));
			config_new.show_size = $(e).attr("gb_show_size");
			// i*100000 ->  HACK to make sure element's id are unique per bridge..., to fix
			bridge = new GoBookBridge_WGO( $(e), null, i*100000, config_new);
		} else {
			// i*100000 ->  HACK to make sure element's id are unique per bridge..., to fix
			bridge = new GoBookBridge_WGO( $(e), null, i*100000, config);
		}
		
		
		// restet onClick for now...
		var play_size = (config.play_size) ? config.play_size : 20;
		if ($(e).attr("gb_play_size")) {
			play_size = $(e).attr("gb_play_size");
		}
		if (config.onProblemRetry) {
			bridge.setOnProblemRetry(config.onProblemRetry);
		}
		if (config.onProblemAnswer) {
			bridge.setOnProblemAnswer(config.onProblemAnswer);
		}
		if (config.onProblemSuccess) {
			bridge.setOnProblemSuccess(config.onProblemSuccess);
		}
		if (config.onProblemFailure) {
			bridge.setOnProblemFailure(config.onProblemFailure);
		}
		if (config.onProblemUnexpected) {
			bridge.setOnProblemUnexpected(config.onProblemUnexpected);
		}
		if (config.onBoardInit) {
			bridge.setOnBoardInit(config.onBoardInit);
		}
		if (config.onGameClick ) {
			bridge.setOnGameClick(config.onGameClick);
		}
		if (config.onBoardClick ) {
			bridge.setOnBoardClick(config.onBoardClick);
		} else {
			// Default : make playable
			bridge.setOnBoardClick ( function(data,x,y,pos){
				$("#boardSidePanel"+data.boardId).show();
				bridge.play(data,play_size);
			});
		}
		
		// only first part (doc is splitted by chapter and h1) is displayed by default
		var from =  $(e).attr("gb_from_part") ?  $(e).attr("gb_from_part") : 0;
		var to = $(e).attr("gb_to_part") ?  $(e).attr("gb_to_part") : 0;
		var language =  $(e).attr("gb_language") ?  $(e).attr("gb_language") : "en";

		engine.executeFromTo( book, bridge, language, from, to);
	});
}

function GoBookBridge_WGO (flow,editor,instanceNumber,config){

	var SIZE = 20;
	var CurrentPosMap = {};
	var DRAW_BOARD = true;
	var AUTO_ENABLE_PROBLEMS = true;

	var flow_saved = false;
	var boardId = 0;
	var _flow = flow;
	var is2Col = false;
	var is3Col = false;
	var row = null; // {left : {html:"",data:data}, right : {html:"",data:data}, center : {html:"",data:data}}
	var onBoardClick = null;
	var onBoardInit = null;
	var onProblemSuccess = null;
	var onProblemFailure = null;
	var onProblemUnexpected = null;
	var onProblemAnswer = null;
	var onGameClick = null;
	var onProblemRetry = null;

	var editMode = false;
	var moveSeq="";

	this.setOnProblemRetry = function(f) {
		onProblemRetry=f;
	};
	this.setOnProblemSuccess = function(f) {
		onProblemSuccess=f;
	};
	this.setOnProblemAnswer = function(f) {
		onProblemAnswer=f;
	};
	this.setOnProblemFailure = function(f) {
		onProblemFailure=f;
	};
	this.setOnProblemUnexpected = function(f) {
		onProblemUnexpected=f;
	};
	this.setOnGameClick = function(f) {
		onGameClick=f;
	};
	this.setOnBoardClick = function(f) {
		onBoardClick=f;
	};
	this.setOnBoardInit = function(f) {
		onBoardInit=f;
	};

	this.toggleEditMode = function() {
		editMode = !editMode;
	};

	this.init = function() {
		if (config) {
			if(config.show_size) {
				SIZE = config.show_size;
			}
		}
		if (instanceNumber) {
			boardId=instanceNumber; // HACK, to fix...
		} else {
			boardId = 0;
		}
		// create a layer for joseki preview
		WGo.Board.drawHandlers.preview = {stone: {
			draw: function(args, board) {
				this.globalAlpha = 0.3;
				if(args.stoneStyle) Board.drawHandlers[args.stoneStyle].stone.draw.call(this, args, board);
				else board.stoneHandler.stone.draw.call(this, args, board);
				this.globalAlpha = 1;
			}
		}};
	};

	/*
	 * API, WGO specific
	 */
	this.appendFigure = function(elem, twoPointsViewPort, base, moves, ab, aw, ca, mn) {
		
		var viewPort = wgo_ViewPort(twoPointsViewPort[0],twoPointsViewPort[1]);
		
		// adjust viewport for coord
		viewPort.top = viewPort.top-0.5;
		viewPort.bottom = viewPort.bottom-0.5;
		viewPort.left = viewPort.left-0.5;
		viewPort.right = viewPort.right-0.5;
		
		var dx = (19-viewPort.right) - viewPort.left;
		
		var data = {elem:elem,boardId:boardId,viewPort:viewPort,dx:dx,base:base,moves:moves,ab:ab,aw:aw, ca:ca, mn:mn};
		
		
		var text ="";
		if (ca) {
			text= ca;
		}
		var id = "elem-"+boardId;

		var res= "<div class='diagram-container' id="+id+">"
				+"	<table>"
				+"		<tr><td>"
				+"			<div class="+elem.cmd+" id='board"+boardId+"'>"+JSON.stringify(twoPointsViewPort)+"</div>"
				
				+"			<div  style='display:none' class='board-next-prev-panel' id='boardPlayPanel"+ boardId+"'>"
				+"				<div style='display:inline-block;float:left' ><button class='btn-play' id='btnPrev"+boardId+"'>&nbsp;&lt;&lt;&nbsp;</button></div>"
				+"				<div style='display:inline-block;float:right'><button class='btn-play' id='btnNext"+boardId+"'>&nbsp;&gt;&gt;&nbsp;</button></div>"
				+"			</div>"
				+"			<div style='display:none' id='boardProbPanel"+ boardId+"'>"
				+"				<div style='display:inline-block;float:left' ><button class='btn-play' id='btnRetry"+boardId+"'>Retry</button></div>"
				+"			</div>"
				+"			<div class='ca' id='boardCaption"+ boardId+"' style='clear:both;'>"+text+"</div>"
				+"		</td></tr><tr>"
				+"		<td valign=top><div id='boardSidePanel"+ boardId+"' style='display:none'><center><button class='btn-play' id='btnReset"+boardId+"'>Reset</button></center></div></td>"
				+"	</tr>"
				+" <tr><td><div class='editMenu' style='display:none' id='boardEdit"+boardId+"'>EDIT </div></td></tr>"
				+"</table>"
				+"</div>";


		if (flow ) {
			if (elem.width) {
				// plusieurs diagrammes sur la même ligne
				var toolbar ="";
				if (row === null) {
					row = {};
				}

				if (!row[elem.align]) {
					row[elem.align] = {};
				}
				
				row[elem.align]["html"] = res;
				row[elem.align].data = data;

				if (elem.width == "half") {
					if (row.left && row.right) {
						res = "<table ><tr><td valign='top'>"+row.left.html+"</td><td valign='top'>"+row.right.html+"</td></tr></table>";
						flow.append("<div>"+res+"</div>");
						initDiagram(row.left.data);
						initDiagram(row.right.data);
						row = null;
					}
				} else if (elem.width == "third") {
					if (row.left && row.right && row.center) {
						res = "<table ><tr><td valign='top'>"+row.left.html+"</td><td valign='top'>"+row.center.html+"</td><td valign='top'>"+row.right.html+"</td></tr></table>";
						flow.append("<div>"+res+"</div>");
						initDiagram(row.left.data);
						initDiagram(row.right.data);
						initDiagram(row.center.data);
						row = null;
					}
				}
			} else {
				// 1 seul diagramme
				flow.append(res);
				initDiagram(data);
			}
		}
		boardId++;
		return res;
	};
		
	/*
	 * API, Non WGO specific
	 */

	this.startToken = function (elem, id) {
		if (flow) {
			if (elem.cmd != "c" && (!elem.width || row===null )) {
				if(!flow_saved) {
					_flow = flow;
					flow_saved = true;
				}
				var cl ="";
				if (editMode) {
					cl = "class='token'";
				}
			
				if (instanceNumber) {
					id = id+"-"+instanceNumber;
				}
				flow.append( "<div id='"+id+"' "+cl+"></div>");
				flow = $("#"+id);
			}
		}
	};

	this.endToken = function (elem) {
		if (flow) {
			if (elem.cmd != "c" && (!elem.width || row===null )) {
				flow = _flow;
				flow_saved = false;
			}
		}
	};
	

	this.appendEditToolBar = function (elem,toolBarPosition) {
		if (editMode) {
			if (flow ) {
				if (elem.cmd != "c"  ) {
					if (elem.width) {
						if (row===null) {
							flow.append( "<div position='"+toolBarPosition+"' class='toolbar'>"+elem.cmd+"</div>");
						}
					} else {
						flow.append( "<div position='"+toolBarPosition+"' class='toolbar'>"+elem.cmd+"</div>");
					}
					
				}
			}
		}
	};

	this.appendHeader = function (elem) {
		closeRowIfNeeded();

		var align = "";
		if (elem.align) {
			align = "style='text-align:"+elem.align+";'";
		}

		var res = "<"+elem.cmd+" "+align+">"+elem.text+"</"+elem.cmd+">";
		if (flow ) {
			flow.append( res);
			
		}
		return res;
	};
	this.appendSection = function (elem) {
		closeRowIfNeeded();
		var res = "<h"+(parseInt(elem.cmd[1],10) +2 )+">"+elem.text+"</h"+(parseInt(elem.cmd[1],10) +2 )+">";
		if (flow) {
			flow.append( res);
			
		}
		return res;

	};
	this.appendParagraph = function (elem) {
		var elemIncluded = closeRowIfNeeded();
		if (! elemIncluded) {
			var align = "";
			if (elem.align) {
				align = "style='text-align:"+elem.align+";'";
			}
			var linkOpenTag="";
			var linkCloseTag="";
			
			// TODO : refactor to remove dependecy on editor.
			if (editor) {
				if (elem.href) {
					var action= "javascript:navTo(\""+elem.href+"\")";
					if (elem.href=="fr") {
						action="javascript:currentLangage=\"fr\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="en") {
						action="javascript:currentLangage=\"en\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="cn") {
						action="javascript:currentLangage=\"cn\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="de") {
						action="javascript:currentLangage=\"de\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="es") {
						action="javascript:currentLangage=\"es\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="it") {
						action="javascript:currentLangage=\"it\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="ja") {
						action="javascript:currentLangage=\"ja\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="ko") {
						action="javascript:currentLangage=\"ko\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="nl") {
						action="javascript:currentLangage=\"nl\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="po") {
						action="javascript:currentLangage=\"po\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="pt") {
						action="javascript:currentLangage=\"pt\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="ru") {
						action="javascript:currentLangage=\"ru\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					} else if (elem.href=="tw") {
						action="javascript:currentLangage=\"tw\";updateUI(editor.getValue(),false,getChapterIndexAtCursor(),getChapterIndexAtCursor());";
					}
					linkOpenTag="<a href='"+action+"'>";
					linkOpenCloseTag="</a>";
				}
			}
			var res = "<p  class='text' "+align+">"+linkOpenTag+elem.text+linkCloseTag+"</p>";
			if (flow) {
				flow.append( res);
			}
			return res;
		}
		return "";
	};
	
	function closeRowIfNeeded() {
		var elemIncuded = false;
		if (row !== null ) {
			res = "<table ><tr>";
			if (row.left ) {
				res+="<td>"+row.left.html+"</td>";
			}
			if (row.center ) {
				res+="<td>"+row.center.html+"</td>";
			}
			if (row.right ) {
				res+="<td>"+row.right.html+"</td>";
			}
			res += "</tr></table>";
			if (flow) {
				flow.append(res);
			}
			// Init diagrams
			if (row.left) {
				initDiagram(row.left.data);
			}
			if (row.right) {
				initDiagram(row.right.data);
			}
			if (row.center) {
				initDiagram(row.center.data);
			}
			row = null;
		}
		return elemIncuded;
	}

	function wgo_ViewPort (leftBottom, rightTop) {
			/*
							p2=T19
					.....o
					......
					o.....
			p1=A1

			-------------

		p1.x = ('A1'[0] to int) = ('A' to int) = 0
		p1.y = ('A1'[1..end] to int) = ('1' to int) = 1
		p2.x = ('T19'[0] to int) = ('T' to int) = 18 (skipping letter I)
		p2.y = ('T19'[1..end] to int) = ('19' to int) = 19

		skip left : p1.x -1
		skip bottom : p1.y - 1
		skip right : 19 - (p2.x to int)
		skip top : 19 - p2.y
		*/

		var p1 = coordAsPoint(leftBottom); // p1 = {x, y}
		var p2 = coordAsPoint(rightTop); // p1 = {x, y}

		if (!p1) {
			p1=coordAsPoint("A1");
		}
		if (!p2) {
			p2=coordAsPoint("T19");
		}
		
		return {
			left : p1.x-1,
			bottom : p1.y -1,
			right : 19- p2.x,
			top : 19 - p2.y
		};
	}
	

	function handleDiagLetter(board, letter, coordList) {
		for (var i = 0; i < coordList.length; i++ ) {
			var coord = coordList[i];
			var pos = coordAsPoint(coord);
			if (pos) {
				board.addObject({x:pos.x-1, y:19-pos.y, type : "LB", text: letter});
			}
		}
	}

	this.initDiagram = function(data) {
		initDiagram(data);
	};

	function initDiagram(data) {
		if (!data) return ;
		var boardId = data.boardId;
		var dx = data.dx;
		var base = data.base;
		var moves = data.moves;
		var ab = data.ab;
		var aw = data.aw;
		var viewPort = data.viewPort;
		var mn = data.mn;
		var elem = data.elem;
		
		$("#boardPlayPanel"+boardId).hide();
		$("#boardSidePanel"+boardId).hide();
		
		
		if (DRAW_BOARD && document.getElementById("board"+boardId)) {
			$("#board"+boardId).html("");
			//
			// Draw the board
			//
			
			var size = SIZE;
			if(size < 8) {size=8;}
			
			var board = new WGo.Board(document.getElementById("board"+boardId), {
				width: dx*size, section: viewPort,
				stoneHandler : WGo.Board.drawHandlers.MONO,
				background : "#ffffff"
			});
					
			
			// WGo.Board.DrawHandler which draws coordinates
			var coordinates = {
				// draw on grid layer
				grid: {
					draw: function(args, board) {
						var ch, t, xright, xleft, ytop, ybottom;
						
						this.fillStyle = "rgba(0,0,0,0.7)";
						this.textBaseline="middle";
						this.textAlign="center";
						this.font = board.stoneRadius+"px "+(board.font || "");
						
						xright = board.getX(-0.75);
						xleft = board.getX(board.size-0.25);
						ytop = board.getY(-0.75);
						ybottom = board.getY(board.size-0.25);
						
						for(var i = 0; i < board.size; i++) {
							ch = i+"A".charCodeAt(0);
							if(ch >= "I".charCodeAt(0)) ch++;
							
							t = board.getY(i);
							this.fillText(board.size-i, xright, t);
							this.fillText(board.size-i, xleft, t);
							
							t = board.getX(i);
							this.fillText(String.fromCharCode(ch), t, ytop);
							this.fillText(String.fromCharCode(ch), t, ybottom);
						}
						
						this.fillStyle = "black";
					}
				}
			};
			
			board.addCustomObject(coordinates);
				

			// letters, exmeple ::fig A="A1 B2 C3" B=Q10R11
			var coordsWithLabels = {}; // so that numbering will not overide the label.
			for (var i = 0;i<26;i++) {
				var upperLetter = String.fromCharCode(65+i); // 'A'
				
				if (elem[upperLetter]) {
					handleDiagLetter(board, upperLetter, elem[upperLetter]);
					for (var j = 0; j< elem[upperLetter].length; j++) {
						coordsWithLabels[elem[upperLetter][j]]=true;
					}
				}
				var lowerLetter =  String.fromCharCode(97+i); // 'a'
				if (elem[lowerLetter]) {
					handleDiagLetter(board, lowerLetter, elem[lowerLetter]);
					for (var j = 0; j< elem[lowerLetter].length; j++) {
						coordsWithLabels[elem[lowerLetter][j]]=true;
					}
				}
			}

			// position attribut, example ::fig A1=X T19=Y
			for (var attr in elem){
				if (attr.match(/([A-Z][0-9]+)/)) {
					var val= elem[attr];
					var pos = coordAsPoint(attr);
					if (pos) {
						coordsWithLabels[attr]=true;
						board.addObject({x:pos.x-1, y:19-pos.y, type : "LB", text: val});
					}
				}
			}
			
			// triangle, circle, square,cross
			if (elem.tr) {
				for (var j = 0; j< elem.tr.length; j++) {
					var pos = coordAsPoint(elem.tr[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, type : "TR"});
				}
			}
			if (elem.cr) {
				for (var j = 0; j< elem.cr.length; j++) {
					var pos = coordAsPoint(elem.cr[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, type : "CR"});
				}
			}
			if (elem.sq) {
				for (var j = 0; j< elem.sq.length; j++) {
					var pos = coordAsPoint(elem.sq[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, type : "SQ"});
				}
			}
			if (elem.ma) {
				for (var j = 0; j< elem.ma.length; j++) {
					var pos = coordAsPoint(elem.ma[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, type : "MA"});
				}
			}
			if (elem.tb) {
				for (var j = 0; j< elem.tb.length; j++) {
					var pos = coordAsPoint(elem.tb[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, c: WGo.B, type : "mini"});
				}
			}
			if (elem.tw) {
				for (var j = 0; j< elem.tw.length; j++) {
					var pos = coordAsPoint(elem.tw[j]);
					board.addObject({x:pos.x-1, y:19-pos.y, c: WGo.W, type : "mini"});
				}
			}
			// 1) pre placed stones
			for (var i in ab) {
				if (editMode) {
					ab[i].type = "preview";
				}
				placeStone(board,  ab[i]);
			}
			for (var i in aw) {
				if (editMode) {
					aw[i].type = "preview";
				}
				placeStone(board,  aw[i]);
			}
			// 2) base
			if (base && base.length>0) {
				for (var i=0; i< base.length; i++) {
					if (editMode) {
						base[i].type = "preview";
					}
					placeStone(board,  base[i]);
				}
			}

			// 3) moves
			if (moves) {
				var startCountingAt = 1;

				if (mn) {
					startCountingAt = parseInt(mn,10);
				} else if (base && base.length>0 && elem.cmd=="fig") {
					startCountingAt = base.length+1;
				}

				for (var i=0; i< moves.length; i++) {
					var label =null;

					if (!coordsWithLabels[moves[i]]) {
						// place number only if there are not allready a label a that coord.

						if (data.elem.cmd==="fig") {
							label = ""+(i+startCountingAt);
						} else {
							// dia start counting at 1 or mn
							label = ""+(i+startCountingAt);
						}
					}
					placeStone(board,  moves[i], label);
				}
			}

			board.removeEventListener("click");
			if (onBoardClick) {
				board.addEventListener("click", function(x, y) {
					var px = x;
					if (px > 7){ px = px+1;} // skip the I letter
					var pos = String.fromCharCode(px+65)+(19-y);
					onBoardClick(data,x,y,pos);
				});
			}
		} // END DRAW_BOARD
		
		if (onBoardInit) {
			onBoardInit(data);
		}
	
		if(AUTO_ENABLE_PROBLEMS) {
			if (elem.cmd==="prb") {
			
				enableGameInteraction(data);
				
			}
		}
	}

	function placeStone(board,  m, label) {
		if (m) {
			var res = [];
			board.addObject(m);
			if (label) {
				board.addObject({x : m.x, y:m.y, type : "LB", text: label});
			}
			// remove any caputred stone from board
			if (res.length > 0) {
				for (var i in res) {
					board.removeObjectsAt(res[i].x,res[i].y);  // remove stone and label
				}
			}
		}
	}
	/* ----------------------------------------------------------------------------------
	 * Game Interaction
	 * ---------------------------------------------------------------------------------- */
	this.play = function(data,size) {
		enableGameInteraction(data,size);
	};

	function enableGameInteraction(data,size) {

		var elem = data.elem;
		var moveSeq ="";
		var boardId = data.boardId;
		//
		// Draw the board
		//
		
		$("#board"+boardId).html("");
		
		if(!size) {
			size = SIZE;
		}
		if(size < 8) {size=8;}
		
		var board = null;
		if (DRAW_BOARD) {
		
			board = new WGo.Board(document.getElementById("board"+boardId), {
				width: data.dx * size, section: data.viewPort,
				stoneHandler : WGo.Board.drawHandlers.NORMAL});

			var coordinates = {
				// draw on grid layer
				grid: {
					draw: function(args, board) {
						var ch, t, xright, xleft, ytop, ybottom;
						
						this.fillStyle = "rgba(0,0,0,0.7)";
						this.textBaseline="middle";
						this.textAlign="center";
						this.font = board.stoneRadius+"px "+(board.font || "");
						
						xright = board.getX(-0.75);
						xleft = board.getX(board.size-0.25);
						ytop = board.getY(-0.75);
						ybottom = board.getY(board.size-0.25);
						
						for(var i = 0; i < board.size; i++) {
							ch = i+"A".charCodeAt(0);
							if(ch >= "I".charCodeAt(0)) ch++;
							
							t = board.getY(i);
							this.fillText(board.size-i, xright, t);
							this.fillText(board.size-i, xleft, t);
							
							t = board.getX(i);
							this.fillText(String.fromCharCode(ch), t, ytop);
							this.fillText(String.fromCharCode(ch), t, ybottom);
						}
						this.fillStyle = "black";
					}
				}
			};
			board.addCustomObject(coordinates);
		}

		var game = new WGo.Game(19);
		CurrentPosMap[boardId] = 0;
		
		/*
		 * Init game at postion
		 */
		var stones = data.base;
		var moves = JSON.parse(JSON.stringify(data.moves));
		var ab = data.ab;
		var aw = data.aw;
		var player = 0;
		if (data.pl==="w") {
			player=1;
		}
		if (ab) {
			for (var i= 0;i< ab.length;i++) {
				var m =ab[i];
				game.addStone(m.x,m.y, m.c);
			}
		}
		if (aw) {
			for (var i= 0;i< aw.length;i++) {
				var m =aw[i];
				game.addStone(m.x,m.y, m.c);
			}
		}
		if (stones) {
			for (var i= 0;i< stones.length;i++) {
				var m = stones[i];
				game.addStone(m.x,m.y, m.c);
			}
		}
		game.validatePosition();
		if (moves && (moves.length > 0)) {
			
			for (var i= 0;i< moves.length;i++) {
				var m = moves[i];
				game.play(m.x,m.y, m.c);
			}
		} else {
			if (elem.pl) {
				 if(elem.pl ==="w") {
				 	game.turn = WGo.W;
				 }
			} else if (elem.go_pl && elem.go_pl ==="w" ) {
				game.turn = WGo.W;
			}
		}
		showGameAtPos(game,board,CurrentPosMap[boardId]);
		// TO TEST
		if (CurrentPosMap[boardId] > 0) {
			var m = moves[CurrentPosMap[boardId]];
			board.addObject({x:m.x, y:m.y, type : "CR"});
		}
		
		/*
		 * Enable prev/next button
		 */
		if (elem.cmd == "prb") {
			$("#boardPlayPanel"+boardId).hide();
			$("#boardProbPanel"+boardId).show();

		} else {
			$("#boardPlayPanel"+boardId).show();
		}	$("#boardNProbPanel"+boardId).hide();
				

		var __game = null; // need a secong game state to handle placing stone freeliy on the board
		$("#btnRetry"+boardId).unbind( "click" );
		$("#btnRetry"+boardId).click(
			function() {
				if (onProblemRetry) {
					onProblemRetry(data);
				}
				enableGameInteraction(data);
			}
		);
		$("#btnReset"+boardId).unbind( "click" );
		$("#btnReset"+boardId).click(
			function() {
				initDiagram(data);
			}
		);
		
		$("#btnPrev"+boardId).unbind( "click" );
		$("#btnPrev"+boardId).click(
			function() {
				var pos = CurrentPosMap[boardId];
				if (pos > 0) {
					pos --;
					showGameAtPos(game,board,pos);
					if (pos > 0) {
						var m = moves[pos-1];
						board.addObject({x:m.x, y:m.y, type : "CR"});
					}
					CurrentPosMap[boardId] = pos;
				} else {
					//alert("Allready at first position");
				}
			}
		);
		$("#btnNext"+boardId).unbind( "click" );
		$("#btnNext"+boardId).click(
			function() {
				var pos = CurrentPosMap[boardId];
				if (pos < game.stack.length-1) {
					pos ++;
					showGameAtPos(game,board,pos);
					CurrentPosMap[boardId] = pos;
					var m = moves[pos-1];
					board.addObject({x:m.x, y:m.y, type : "CR"});
				} else {
					//alert("Allready at end position");
				}
			}
		);

		/*
		 * Place own stone at any position...
		 */
		
		if (board) {
			board.removeEventListener("click");
			if (onGameClick) {
				board.addEventListener("click", function(x, y) {
					var px = x;
					if (px > 7){ px = px+1;} // skip the I letter
					var pos = String.fromCharCode(px+65)+(19-y);
					onGameClick(data,x,y,pos,game);
				});
			
			} else {
				board.addEventListener("click", function(x, y) {

					var pos = CurrentPosMap[boardId] +1;
					if (pos > 0)  {
						moves[pos-1] = {x:x, y:y};
					}
					var size = game.stack.length
					while (game.stack.length > pos) {
						game.popPosition();
					}
					
					var res = game.play(x, y);
					showGameAtPos(game,board,pos);
					
					board.addObject({x:x, y:y, type : "CR"});
					
					
					
					if (elem.cmd=="prb") {
						var coord = pointAsCoord(x,y);
						moveSeq += coord;
						// FOR PROBLEM MODE
						
						var answer = GBProblemHelper.answerMove(elem,moveSeq,coord);
						if(onProblemAnswer) {
							onProblemAnswer(data,answer);
						}
						if (answer.pos==="" ) {
							showGameAtPos(game,board,pos);
							board.addObject({x:x, y:y, type : "CR"});

							switch (answer.path) {
								case "correct" :
									if (onProblemSuccess) {
										onProblemSuccess(data,answer);
									}
									break;
								case "wrong" :
									if (onProblemFailure) {
										onProblemFailure(data,answer);
									}
									break;
								default :
									if (onProblemUnexpected) {
										onProblemUnexpected(data,answer);
									} 
							}
						} else {
							// we have got an answer, so display it
							moveSeq += answer.pos;
							var point = coordAsPoint(answer.pos);
							if(point) {
								x = point.x-1;
								y = 19-point.y;
								res = game.play(x,y);
								pos++;
							}
							showGameAtPos(game,board,pos);
							board.addObject({x:x, y:y, type : "CR"});

						}
						
					} // end prb
					
					
					CurrentPosMap[boardId] = pos;
				}); // end addEventListener
			}
		}
	}
	/* ----------------------------------------------------------------------------------
	 * End enableGameInteraction
	 * ---------------------------------------------------------------------------------- */

	function showGameAtPos(game,board,pos) {
		if (!pos) {
			pos = 0
		}
		// debug
		/*
		var text="";
		for(var x = 0;x< 19;x++) {
			for(var y = 0;y< 19;y++) {
				var stone = game.stack[pos].schema[y*19+x];
				var p = ".";
				if (stone == 1) {
					p = "X";
				} else if (stone==-1) {
					p = "O";
				}
				text +=p+" ";
			}
			text +="\n";
		}
		*/
		//console.log(text);
		// end debug

		if (board) {
			board.removeAllObjects();
		}
		for(var y = 0;y< 19;y++) {
			for(var x = 0;x< 19;x++) {
				var stone = game.stack[pos].schema[y+x*19];
				
				if (stone !== 0) {
					if (board) {
						board.addObject({ x: x, y: y, c: stone  });
					}	
				}
			}
		}
	}


	function pointAsCoord(x,y) {
		if (x > 7){ x = x+1;} // skip the I letter
		var coord = String.fromCharCode(x+65)+(19-y);
		return coord 
	}
	
	function coordAsPoint (coord) {
		// coord : 'A1' (left/Bottom) to 'T19' (right/Top)
		// return a point {x,y} from 1,1 to 19,19
		if (coord) {
			var x = coord.charCodeAt(0) - 65 + 1;
			var y = parseInt(coord.substring(1),10);

			if (x>=9) {
				x = x-1; // skip letter I
			}
			return {x:x,y:y};
		}
		return null;
		
	}

/*
 * End of GoBookBridge_WGO object
 */
}