function GoBookEngine() {

	// state
	var elementIdMap = {};
	var currentGoElemForChapter = null;
	var currentDiagElem = null;
	
	this.executeFromTo = function(book, bridge, langage , from ,to) {
		
		bridge.init();
		var currentChapter =0;
		var res= "";
		currentGoElemForChapter = null;
		var toolBarPosition = 0;
		var tokenCpt = 0;
		for (var i = 0;i< book.length;i++) {
			var elem = book[i];

			var cmd = elem.cmd;
			var elemLangages =null;

			if (cmd && cmd.indexOf("_") > 0) {
				
				var cmdLangagesString = elem.cmd.substr(elem.cmd.indexOf("_")+1);
				elemLangages = cmdLangagesString.split("_");
				cmd = cmd.substr(0,elem.cmd.indexOf("_"));
				elem.cmd = cmd;
			}
			
			// ----------------------------------------------------------------------------
			// Langage filter for token

			// cn (Simplified Chinese), de(German), en (English, default), es (Spanish), fr (French), it (Italian), ja (Japanese),
			// ko (Korean), nl (Dutch), po (Polish; not pl due to conflict with player), pt (Portuguese),
			// ru (Russian), tw (Traditional Chinese)

			var langages = ["cn", "de", "en", "es", "fr", "it", "ja", "ko", "nl", "po", "pt", "ru", "tw" ];

			if(langages.indexOf(cmd)>-1) {
				// this a paragraph in a specified langage like '::en an english text...'
				if (langage !== cmd) {
					continue; // not the current langage, just ignore this token
				} else {
					cmd = "p"; //::en ::fr etc are handled like ::p
					elem.cmd = cmd;
				}
			}
			if (elemLangages !== null) {
				if(elemLangages.indexOf(langage) < 0) {
					continue; // token is not for the current langage, just ignore this token
				}
			}

			// end langage filter for token
			// ----------------------------------------------------------------------------
			if  (elem.cmd=="chapter") {
				currentGoElemForChapter = null;
			}
			

			if  (elem.cmd=="chapter" || elem.cmd=="h1") {
				currentChapter ++;
			}

			if ((currentChapter >= from) &&(currentChapter <=to)) {

			
				res += bridge.startToken(elem, tokenCpt);
				elem.position = tokenCpt;
				tokenCpt ++;
				//console.log("Processing node : "+JSON.stringify(book[i],null,2));

				elem.text = formatText(book[0], elem, langage);
				var cmd = elem.cmd;
				
				if (cmd[0] =="h") { // h1... h5
					res+=bridge.appendHeader(elem);
				} else if (cmd[0] =="s") { // s1... s2 ==> convert to h1 or h2
					res+=bridge.appendSection(elem);
				} else if (cmd =="p") {
					res+=bridge.appendParagraph(elem);
					
				} else if (cmd=="chapter") {
					currentGoElemForChapter = null; // reset
					currentDiagElem = null; // reset
				
				} else if (cmd=="go") {
					currentGoElemForChapter = __build(null,elem); // save current go data
					currentDiagElem = null; // reset
					
				} else if (cmd=="fig" || cmd=="dia" || cmd=="var" || cmd=="prb") {
					
					if (cmd=="prb") {
						if (currentGoElemForChapter) {
							elem.go_mv = currentGoElemForChapter.mv_seq;
							elem.go_pl = currentGoElemForChapter.pl;
						}
					}
					res+=drawBoard(bridge, cmd, elem);
				}

				bridge.appendEditToolBar(elem, toolBarPosition);
				toolBarPosition++;

				res += bridge.endToken(elem);

			} else {
				// console.log("Skipping node : "+JSON.stringify(book[i],null,2));
			}
		}
		return res;
	};


	/*
	 * call executeFromTo from start to end...
	 */

	this.execute = function(book, bridge, language) {
		this.executeFromTo(book, bridge, language, 0, book.length);
	};


	function drawBoard(bridge, type, elem) {
		var base = __getBase(elem);
		var st = __build(base,elem);
		currentDiagElem = st;
		return bridge.appendFigure(elem, st.vw, st.seq_before_at, st.seq_after_at, st.ab, st.aw, st.ca, st.mn);
	}

	function __getBase(elem) {
		if (elem.base) {
			if (elem.base ==="none") {
				return null;
			} else {
				return elementIdMap[elem.base];
			}
		} else {
			if (elem.cmd==="var") {
				if (currentDiagElem) {
					return currentDiagElem;
				}
			}
		}
		return currentGoElemForChapter; 
	}

	function __newBase(elem) {
		return  {
			mn : (elem.mn) ? elem.mn : 1,
			vw: (elem.vw) ? elem.vw : ["A1","T19"],
			ca: (elem.ca) ? elem.ca : "",
			ab :[] ,
			aw :[] ,
			seq_before_at : [],
			seq_after_at :[]
		};
	}

	function __build(base,elem) {
		var c = 1;
		if (!base) {
			// pas de base (base=none ou pas de parent)
			// ex : ::fig mv=A1A2A3A4 at=2
			base = __newBase(elem);
			if(elem.mv) {
				base.mv_seq = elem.mv.join(""); // just to keep the orignal mv seq. usefull for problems
			}
			if(elem.pl) {
				base.pl = elem.pl; // usefull for problems
			}

			if (elem.pl ==="w") {c = -1;}
			base.pl_seq_after_at = c;
			if (elem.mv) {
				for(var i = 0;i<elem.mv.length;i++) {
					var p = coordAsPoint(elem.mv[i],c);
					if (p) {
						base.seq_after_at.push(p);
					}
					c *= -1;
				}
			}
			if (elem.ab) {
				for(var b = 0;b<elem.ab.length;b++) {
					base.ab.push(coordAsPoint(elem.ab[b],1));
				}
			}
			if (elem.aw) {
				for(var w = 0;w<elem.aw.length;w++) {
					base.aw.push(coordAsPoint(elem.aw[w],-1));
				}
			}
			if (elem.id) {
				elementIdMap[elem.id] = base;
			}
			return base;

		} else {
			// there is a base
			
			var newBase =  JSON.parse(JSON.stringify(base));
			
			var seq =  newBase.seq_before_at.concat( newBase.seq_after_at);
			var	at = null;
			if (elem.at ) {
				at = (elem.at=="end") ? seq.length+1 : parseInt(elem.at,10);
			}


			if (elem.vw) { newBase.vw = elem.vw; }
			if (elem.ca) { newBase.ca = elem.ca; }
			if (elem.mn) {
				newBase.mn = elem.mn;
			} else {
				
				if (at && elem.cmd !="dia") {
					newBase.mn = at;
				} else {
					if (elem.cmd=="var"   ) {
						newBase.mn = seq.length;
					}
				}
			
			}
			
			if (elem.ab) {
				for(var nb = 0; nb<elem.ab.length; nb++) {
					newBase.ab.push(coordAsPoint(elem.ab[nb],1));
				}
			}
			if (elem.aw) {
				for(var nw = 0; nw<elem.aw.length; nw++) {
					newBase.aw.push(coordAsPoint(elem.aw[nw],-1));
				}
			}

			// The move sequence can be specified either
			//    1) explicitly with mv with or without at
			// or 2) by using at and to in combination to designate a range of moves in the data this element is based on.
			// (The two ways of specifying a sequence are mutually exclusive.)
		
			
			
			if(elem.mv) {
				
				//
				// 1) explicitly with mv with or without at
				//
				if (at) {
					newBase.seq_before_at = seq.slice(0,at-1);
				}
				newBase.seq_after_at = [];
				
				if (elem.pl) {
					// forced color ?
					c = (elem.pl =="b") ?  1 :-1;
				} else if ( newBase.seq_before_at.length > 0) {
					// if not just continue the sequence
					c = newBase.seq_before_at[ newBase.seq_before_at.length-1].c *-1;
				}

				for(var m = 0; m <elem.mv.length; m++) {
					var p = coordAsPoint(elem.mv[m],c);
					if (p) {
						newBase.seq_after_at.push(p);
					}
					c *=-1;
				}
							
			} else if (at && elem.to){
				//
				//  2) by using AT and TO in combination to designate a range of moves in the data this element is based on.
				//
				var	to = (elem.to=="end") ? seq.length+1 :  parseInt(elem.to,10);
				newBase.seq_before_at = seq.slice(0,at-1);
				newBase.seq_after_at =	seq.slice(at-1, to);

			} else  {
				// just at ?
				newBase.seq_before_at = (at) ? seq.slice(0,at-1) : [];
				newBase.seq_after_at = [];
			}

			if (elem.id) {
				elementIdMap[elem.id] = newBase;
			}

			return newBase;
		}
	}

	function coordAsPoint (coord,color) {
		if (!coord) return null;
		if ((coord.toUpperCase() =="PASS") || (coord.toUpperCase() =="TENUKI")) {
			return null;
		}

		// coord : 'A1' (left/Bottom) to 'T19' (right/Top)
		// return a point {x,y} from 1,1 to 19,19
		res = null;
		if (coord) {
			var x = coord.charCodeAt(0) - 65 ;
			var y = 19-parseInt(coord.substring(1),10);

			if (x>=9) {
				x = x-1; // skip letter I
			}
			if (color) {
				res= {coord : coord, x:x, y:y, c:color};
			} else {
				res= {coord : coord, x:x, y:y};
			}
		}
		return res;
	}

	function formatText(bookElem, elem,langage) {
		if (elem.text === null) {
			return "";
		}
		
		var text_buffer="";
		var italic = false;
		var bold = false;
		var underline = false;
		
		for (var i = 0;i< elem.text.length-1 ;i++) {
			
			var check =elem.text[i]+elem.text[i+1];
			if (check =="__") {
				if (italic) {
					text_buffer+="</i>";
					italic = false;
				} else {
					text_buffer+="<i>";
					italic = true;
				}
			} else if (check =="**") {
				if (bold) {
					text_buffer+="</b>";
					bold = false;
				} else {
					text_buffer+="<b>";
					bold = true;
				}
			} else if (check =="==") {
				if (underline) {
					text_buffer+="</u>";
					underline = false;
				} else {
					text_buffer+="<u>";
					underline = true;
				}
			}
			
			text_buffer+= elem.text[i];
			
		}
		text_buffer+=elem.text[elem.text.length-1];
		
		// TODO : common text like ^T  or #btp# -> "Black to play." see page 16 of documentation

		// TODO, not perfect, will not find attribute with multiple langage like "tile_en_fr"
		var title =  (bookElem["title_"+langage]) ? bookElem["title_" + langage] : bookElem["title"];
		var author =  (bookElem["author_"+langage]) ? bookElem["author_" + langage] : bookElem["author"];
		var publisher =  (bookElem["publisher_"+langage]) ? bookElem["publisher_" + langage] : bookElem["publisher"];
		var subtitle =  (bookElem["subtitle_"+langage]) ? bookElem["subtitle_" + langage] : bookElem["subtitle"];
		var copyright =  (bookElem["copyright_"+langage]) ? bookElem["copyright_" + langage] : bookElem["copyright"];

		return text_buffer
			.replace(/href=\"cn\"/g," href=\"#\" onclick='javascript:currentLangage=\"cn\";updateUI(editor.getValue(),false)'")   // TODO : no javascript here... better to have bridge.getChangeLanguageAction("fr")
			.replace(/href=\"de\"/g," href=\"#\" onclick='javascript:currentLangage=\"de\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"en\"/g," href=\"#\" onclick='javascript:currentLangage=\"en\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"es\"/g," href=\"#\" onclick='javascript:currentLangage=\"es\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"fr\"/g," href=\"#\" onclick='javascript:currentLangage=\"fr\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"it\"/g," href=\"#\" onclick='javascript:currentLangage=\"it\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"ja\"/g," href=\"#\" onclick='javascript:currentLangage=\"ja\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"ko\"/g," href=\"#\" onclick='javascript:currentLangage=\"ko\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"nl\"/g," href=\"#\" onclick='javascript:currentLangage=\"nl\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"po\"/g," href=\"#\" onclick='javascript:currentLangage=\"po\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"pt\"/g," href=\"#\" onclick='javascript:currentLangage=\"pt\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"ru\"/g," href=\"#\" onclick='javascript:currentLangage=\"ru\";updateUI(editor.getValue(),false)'")
			.replace(/href=\"tw\"/g," href=\"#\" onclick='javascript:currentLangage=\"tw\";updateUI(editor.getValue(),false)'")
			.replace(/href=([a-zA-Z0-9-_]+)/g," href=\"#\" onclick='javascript:navTo(\"$1\")'")
			.replace(/[=][=]/g,"")
			.replace(/__/g,"")
			.replace(/[*][*]/g,"")
			.replace(/\n/g,"<br>")
			.replace(/#title#/g, title)
			.replace(/#author#/g, author)
			.replace(/#publisher#/g, publisher)
			.replace(/#subtitle#/g, subtitle)
			.replace(/#copyright#/g, copyright);
			
	}

	
}